package com.chuanxia.mcp.controller;

import com.chuanxia.mcp.aop.Login;
import com.chuanxia.mcp.bean.dto.DomainOperatorBindDto;
import com.chuanxia.mcp.bean.dto.OperatorQueryDto;
import com.chuanxia.mcp.bean.model.CdnOperator;
import com.chuanxia.mcp.bean.vo.Result;
import com.chuanxia.mcp.service.CdnOperatorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * @author liyuntian
 */
@RestController
@Api("cdn运营商管理")
@RequestMapping("cdnOperator")
@RequiredArgsConstructor
public class CdnOperatorController {
    private final CdnOperatorService operatorService;

    @ApiOperation("新增运营商")
    @Login
    @PostMapping("saveOperator")
    public Result saveOperator(@RequestBody CdnOperator operator) {
        return operatorService.saveOperator(operator);
    }

    @ApiOperation("修改运营商")
    @Login
    @PostMapping("updateOperator")
    public Result updateOperator(@RequestBody CdnOperator operator) {
        return operatorService.updateOperator(operator);
    }

    @GetMapping("getList")
    @ApiOperation("获取所有运营商列表(不分页)")
    @Login
    public Result getList() {
        return Result.success(operatorService.list());
    }

    @PostMapping("bindCdn")
    @ApiOperation("域名绑定运营商")
    @Login
    public Result bindCdn(@RequestBody DomainOperatorBindDto bindDto) {
        return operatorService.bindCdn(bindDto);
    }

    @PostMapping("listPage")
    @ApiOperation("运营商分页列表")
    @Login
    public Result listPage(@RequestBody OperatorQueryDto queryDto) {
        return operatorService.listPage(queryDto);
    }

}
