package com.chuanxia.mcp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chuanxia.mcp.bean.dto.DomainBindDto;
import com.chuanxia.mcp.bean.dto.DomainOperatorBindDto;
import com.chuanxia.mcp.bean.dto.OperatorQueryDto;
import com.chuanxia.mcp.bean.model.CdnDomain;
import com.chuanxia.mcp.bean.model.CdnDomainOperator;
import com.chuanxia.mcp.bean.model.CdnOperator;
import com.chuanxia.mcp.bean.model.SystemUser;
import com.chuanxia.mcp.bean.vo.Result;
import com.chuanxia.mcp.exception.ResultException;
import com.chuanxia.mcp.mapper.CdnDomainMapper;
import com.chuanxia.mcp.mapper.CdnOperatorMapper;
import com.chuanxia.mcp.service.CdnDomainOperatorService;
import com.chuanxia.mcp.service.CdnOperatorService;
import com.chuanxia.mcp.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class CdnOperatorServiceImpl extends ServiceImpl<CdnOperatorMapper, CdnOperator> implements CdnOperatorService {
    private final CdnOperatorMapper operatorMapper;
    private final CdnDomainMapper domainMapper;
    private final UserService userService;
    private final CdnDomainOperatorService domainOperatorService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result saveOperator(CdnOperator operator) {
        SystemUser loginUser = userService.getLoginUser();
        operator.setCreateBy(loginUser.getId());
        operatorMapper.insert(operator);
        return Result.success();
    }

    @Override
    public Result updateOperator(CdnOperator operator) {
        SystemUser loginUser = userService.getLoginUser();
        operator.setUpdateBy(loginUser.getId());
        operatorMapper.updateById(operator);
        return Result.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result bindCdn(DomainOperatorBindDto bindDto) {
        Integer domainId = bindDto.getDomainId();
        CdnDomain domain = domainMapper.selectById(domainId);
        if (domain == null) {
            throw new ResultException(500, "域名id不存在");
        }
        List<Integer> operatorIds = bindDto.getOperatorIds();
        if (CollectionUtil.isEmpty(operatorIds)) {
            throw new ResultException(500, "运营商id不能为空");
        }
        SystemUser loginUser = userService.getLoginUser();
        List<CdnDomainOperator> arr = new ArrayList<>(operatorIds.size());
        for (Integer operatorId : operatorIds) {
            CdnDomainOperator domainOperator = new CdnDomainOperator();
            domainOperator.setOperatorId(operatorId);
            domainOperator.setDomainId(domainId);
            domainOperator.setCreateBy(loginUser.getId());
            arr.add(domainOperator);
        }
        LambdaQueryWrapper<CdnDomainOperator> del = Wrappers.lambdaQuery();
        del.eq(CdnDomainOperator::getDomainId, domainId);
        domainOperatorService.remove(del);
        domainOperatorService.saveBatch(arr);
        return Result.success();
    }

    @Override
    public Result listPage(OperatorQueryDto queryDto) {
        return null;
    }
}
